//管理用户信息、登录状态、角色权限等
import { defineStore } from 'pinia'
import Taro from '@tarojs/taro'
import { userAPI } from '@/services/api/user'
import { ref } from 'vue'

export type UserRole = 'player' | 'hoster'

// 定义API返回类型接口
interface OpenIdResult {
  openid: string;
  [key: string]: any;
}

interface UserState {
  role: UserRole
  nickname: string
  avatar: string
  openid: string
  roomId: string | null
  currentGameId: string | null
  isRegistered: boolean
  currentRoom: string | null
}

export const useUserStore = defineStore('user', () => {
  const openid = ref('')
  const nickname = ref('')
  const avatar = ref('')
  const role = ref<'player' | 'hoster'>('player')
  const isRegistered = ref(false)
  const currentRoom = ref<string | null>(null)
  const currentGameId = ref<string | null>(null)
  
  // 初始化用户信息
  async function initUser() {
    // 从本地存储加载用户信息
    const userInfo = Taro.getStorageSync('userInfo')
    if (userInfo) {
      try {
        const info = JSON.parse(userInfo)
        openid.value = info.openid || ''
        nickname.value = info.nickname || ''
        avatar.value = info.avatar || ''
        role.value = info.role || 'player'
        isRegistered.value = !!info.nickname && !!info.openid
        
        // 如果有房间ID，恢复
        if (info.currentRoom) {
          currentRoom.value = info.currentRoom
        }
        
        return true
      } catch (e) {
        console.error('解析用户信息失败:', e)
      }
    }
    
    // 如果没有本地信息，尝试静默登录
    return silentLogin()
  }
  
  // 静默登录 - 仅获取openid
  async function silentLogin() {
    try {
      const loginRes = await Taro.login()
      if (loginRes.code) {
        // 修复: 确保返回类型正确
        const result = await userAPI.getOpenId(loginRes.code) as OpenIdResult
        if (result && result.openid) {
          openid.value = result.openid
          
          // 保存到本地
          Taro.setStorageSync('userInfo', JSON.stringify({
            openid: result.openid,
            role: role.value
          }))
          
          return true
        }
      }
      return false
    } catch (error) {
      console.error('静默登录失败:', error)
      return false
    }
  }
  
  // 注册用户 - 获取用户信息
  async function registerUser() {
    try {
      // 微信环境使用getUserProfile
      if (process.env.TARO_ENV === 'weapp') {
        const profileRes = await Taro.getUserProfile({
          desc: '用于完善用户资料'
        })
        
        if (profileRes.userInfo) {
          nickname.value = profileRes.userInfo.nickName
          avatar.value = profileRes.userInfo.avatarUrl
          isRegistered.value = true
          
          // 保存到本地
          const userInfo = {
            openid: openid.value,
            nickname: profileRes.userInfo.nickName,
            avatar: profileRes.userInfo.avatarUrl,
            role: role.value,
            currentRoom: currentRoom.value
          }
          Taro.setStorageSync('userInfo', JSON.stringify(userInfo))
          
          // 同步到云端
          if (process.env.TARO_ENV === 'weapp' && Taro.cloud) {
            try {
              // 修复: 使用正确的云初始化检查方式
              // 注意: Taro.cloud.init()会返回void，不需要检查inited属性
              await Taro.cloud.init({
                env: (Taro.cloud as any).DYNAMIC_CURRENT_ENV
              })
              
              // 注意：云开发数据库操作
              const db = Taro.cloud.database()
              const userQuery = await db.collection('users').where({
                openid: openid.value
              }).get()
              
              // 如果用户已存在，更新信息
              if (userQuery.data && userQuery.data.length > 0) {
                // 修复: 强制类型转换
                const docId = userQuery.data[0]._id as string
                await db.collection('users').doc(docId).update({
                  data: {
                    nickname: nickname.value,
                    avatar: avatar.value,
                    updateTime: db.serverDate()
                  }
                })
              } else {
                // 创建新用户
                await db.collection('users').add({
                  data: {
                    openid: openid.value,
                    nickname: nickname.value,
                    avatar: avatar.value,
                    role: role.value,
                    createTime: db.serverDate(),
                    updateTime: db.serverDate()
                  }
                })
              }
            } catch (cloudError) {
              console.error('同步用户信息到云端失败:', cloudError)
              // 继续执行，即使云同步失败
            }
          }
          
          return true
        }
      } else {
        // 非微信环境模拟注册
        nickname.value = '模拟用户_' + Date.now().toString().slice(-4)
        avatar.value = 'https://example.com/avatar.jpg'
        isRegistered.value = true
        
        // 保存到本地
        const userInfo = {
          openid: openid.value,
          nickname: nickname.value,
          avatar: avatar.value,
          role: role.value,
          currentRoom: currentRoom.value
        }
        Taro.setStorageSync('userInfo', JSON.stringify(userInfo))
        
        return true
      }
      
      return false
    } catch (error) {
      console.error('用户注册失败:', error)
      return false
    }
  }
  
  // 设置用户角色
  function setRole(newRole: 'player' | 'hoster') {
    role.value = newRole
    
    // 更新本地存储
    const userInfo = Taro.getStorageSync('userInfo')
    if (userInfo) {
      try {
        const info = JSON.parse(userInfo)
        info.role = newRole
        Taro.setStorageSync('userInfo', JSON.stringify(info))
      } catch (e) {
        console.error('更新用户角色失败:', e)
      }
    }
  }
  
  // 设置当前房间
  function setCurrentRoom(roomId: string) {
    currentRoom.value = roomId
    
    // 更新本地存储
    const userInfo = Taro.getStorageSync('userInfo')
    if (userInfo) {
      try {
        const info = JSON.parse(userInfo)
        info.currentRoom = roomId
        Taro.setStorageSync('userInfo', JSON.stringify(info))
      } catch (e) {
        console.error('更新当前房间失败:', e)
      }
    }
  }
  
  // 检查是否已注册
  function checkIsRegistered() {
    return isRegistered.value
  }
  
  // 退出登录
  function logout() {
    openid.value = ''
    nickname.value = ''
    avatar.value = ''
    isRegistered.value = false
    currentRoom.value = null
    
    // 清除本地存储
    Taro.removeStorageSync('userInfo')
  }
  
  // 检查用户登录状态
  async function checkLoginStatus() {
    try {
      // 从本地存储获取用户信息
      const userInfo = Taro.getStorageSync('userInfo')
      if (userInfo) {
        const info = JSON.parse(userInfo)
        openid.value = info.openid || ''
        nickname.value = info.nickname || ''
        avatar.value = info.avatar || ''
        isRegistered.value = !!info.openid && !!info.nickname
        return true
      }
      
      // 如果没有本地信息，尝试静默登录获取openid
      const loginRes = await Taro.login()
      if (loginRes.code) {
        // 修复: 确保返回类型正确
        const result = await userAPI.getOpenId(loginRes.code) as OpenIdResult
        if (result && result.openid) {
          openid.value = result.openid
          // 保存到本地
          Taro.setStorageSync('userInfo', JSON.stringify({ openid: result.openid }))
          return true
        }
      }
      
      return false
    } catch (error) {
      console.error('检查登录状态失败:', error)
      return false
    }
  }
  
  // 设置当前游戏
  function setCurrentGame(gameId: string | null) {
    currentGameId.value = gameId  // 修复: 使用ref value
  }
  
  // 获取用户信息
  async function getUserInfo() {
    try {
      // 尝试从本地存储获取角色
      const roleValue = Taro.getStorageSync('userRole') as UserRole | ''
      if (roleValue) {
        role.value = roleValue
      }
      
      // 获取用户信息
      const userData = await userAPI.getUserInfo()
      if (userData) {
        nickname.value = userData.nickname || '游客'
        avatar.value = userData.avatar || ''
        openid.value = userData.openid || ''
      }
    } catch (error) {
      console.error('获取用户信息失败:', error)
    }
  }
  
  // 重置用户角色到玩家
  function resetToPlayer() {
    role.value = 'player'  // 修复: 使用ref value
    Taro.setStorageSync('userRole', 'player')
  }
  
  // 清除房间和游戏信息
  function clearGameSession() {
    currentRoom.value = null  // 修复: 使用ref value
    currentGameId.value = null  // 修复: 使用ref value
  }
  
  return {
    openid,
    nickname,
    avatar,
    role,
    isRegistered,
    currentRoom,
    currentGameId,
    initUser,
    registerUser,
    setRole,
    setCurrentRoom,
    checkIsRegistered,
    logout,
    checkLoginStatus,
    setCurrentGame,
    getUserInfo,
    resetToPlayer,
    clearGameSession
  }
}) 